from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.pool_service_get_pool_number_lookup_response_200 import (
    PoolService_GetPoolNumberLookupResponse_200,
)
from ...models.pool_service_get_pool_number_lookup_response_401 import (
    PoolService_GetPoolNumberLookupResponse_401,
)
from ...models.pool_service_get_pool_number_lookup_response_404 import (
    PoolService_GetPoolNumberLookupResponse_404,
)
from ...models.pool_service_get_pool_number_lookup_response_500 import (
    PoolService_GetPoolNumberLookupResponse_500,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    pool_number: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/pools/pool_number_lookup".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    params: Dict[str, Any] = {
        "poolNumber": pool_number,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        PoolService_GetPoolNumberLookupResponse_200,
        PoolService_GetPoolNumberLookupResponse_401,
        PoolService_GetPoolNumberLookupResponse_404,
        PoolService_GetPoolNumberLookupResponse_500,
    ]
]:
    if response.status_code == 200:
        response_200 = PoolService_GetPoolNumberLookupResponse_200.from_dict(
            response.json()
        )

        return response_200
    if response.status_code == 401:
        response_401 = PoolService_GetPoolNumberLookupResponse_401.from_dict(
            response.json()
        )

        return response_401
    if response.status_code == 404:
        response_404 = PoolService_GetPoolNumberLookupResponse_404.from_dict(
            response.json()
        )

        return response_404
    if response.status_code == 500:
        response_500 = PoolService_GetPoolNumberLookupResponse_500.from_dict(
            response.json()
        )

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        PoolService_GetPoolNumberLookupResponse_200,
        PoolService_GetPoolNumberLookupResponse_401,
        PoolService_GetPoolNumberLookupResponse_404,
        PoolService_GetPoolNumberLookupResponse_500,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    pool_number: Union[Unset, str] = UNSET,
) -> Response[
    Union[
        PoolService_GetPoolNumberLookupResponse_200,
        PoolService_GetPoolNumberLookupResponse_401,
        PoolService_GetPoolNumberLookupResponse_404,
        PoolService_GetPoolNumberLookupResponse_500,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        pool_number=pool_number,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    pool_number: Union[Unset, str] = UNSET,
) -> Optional[
    Union[
        PoolService_GetPoolNumberLookupResponse_200,
        PoolService_GetPoolNumberLookupResponse_401,
        PoolService_GetPoolNumberLookupResponse_404,
        PoolService_GetPoolNumberLookupResponse_500,
    ]
]:
    """


    Args:
        pool_number: None

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        pool_number=pool_number,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    pool_number: Union[Unset, str] = UNSET,
) -> Response[
    Union[
        PoolService_GetPoolNumberLookupResponse_200,
        PoolService_GetPoolNumberLookupResponse_401,
        PoolService_GetPoolNumberLookupResponse_404,
        PoolService_GetPoolNumberLookupResponse_500,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        pool_number=pool_number,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    pool_number: Union[Unset, str] = UNSET,
) -> Optional[
    Union[
        PoolService_GetPoolNumberLookupResponse_200,
        PoolService_GetPoolNumberLookupResponse_401,
        PoolService_GetPoolNumberLookupResponse_404,
        PoolService_GetPoolNumberLookupResponse_500,
    ]
]:
    """  """

    return (
        await asyncio_detailed(
            client=client,
            pool_number=pool_number,
        )
    ).parsed
