from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.pool_service_get_summaries_json_body import (
    PoolService_GetSummariesJsonBody,
)
from ...models.pool_summaries import PoolSummaries
from ...models.pool_summaries_error import PoolSummariesError
from ...models.pool_summaries_not_found import PoolSummariesNotFound
from ...models.pool_summaries_unauthorized import PoolSummariesUnauthorized
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: PoolService_GetSummariesJsonBody,
) -> Dict[str, Any]:
    url = "{}/api/v1/pools/summaries".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        PoolSummaries,
        PoolSummariesUnauthorized,
        PoolSummariesNotFound,
        PoolSummariesError,
    ]
]:
    if response.status_code == 200:
        response_200 = PoolSummaries.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = PoolSummariesUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = PoolSummariesNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = PoolSummariesError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        PoolSummaries,
        PoolSummariesUnauthorized,
        PoolSummariesNotFound,
        PoolSummariesError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: PoolService_GetSummariesJsonBody,
) -> Response[
    Union[
        PoolSummaries,
        PoolSummariesUnauthorized,
        PoolSummariesNotFound,
        PoolSummariesError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: PoolService_GetSummariesJsonBody,
) -> Optional[
    Union[
        PoolSummaries,
        PoolSummariesUnauthorized,
        PoolSummariesNotFound,
        PoolSummariesError,
    ]
]:
    """


    Args:

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: PoolService_GetSummariesJsonBody,
) -> Response[
    Union[
        PoolSummaries,
        PoolSummariesUnauthorized,
        PoolSummariesNotFound,
        PoolSummariesError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: PoolService_GetSummariesJsonBody,
) -> Optional[
    Union[
        PoolSummaries,
        PoolSummariesUnauthorized,
        PoolSummariesNotFound,
        PoolSummariesError,
    ]
]:
    """  """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed
